window.Gleam.initSearch([{"doc":"embeds","title":"embeds","content":"# embeds\n\n[![Package Version](https://img.shields.io/hexpm/v/embeds)](https://hex.pm/packages/embeds)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/embeds/)\n\nA Gleam code generator, allowing you to generate constants and zero-argument functions based on files, environment variables, or any other source you&#39;d like.\n\nIt provides simple command-line tools for basic use cases, but allows you to hook into most parts of the generation process. This can be used to [Automatically generate Lustre code](./examples/lustre-icons.html) for SVG icons, for example.\n\n## Getting started\n\nInstall `embeds` as a dev dependency:\n\n```sh\ngleam add --dev embeds\n```\n\nRun the generators:\n\n```sh\ngleam run -m embeds/files -- --help\ngleam run -m embeds/env -- --help\n```\n\n## Why?\n\nPlease keep in mind that embedding files into Gleam is often not necessary when running on the Erlang platform.\n\nErlang provides `priv` directory, which will be bundled with your application code when building an Erlang shipment. When using [wisp](https://hexdocs.pm/wisp/index.html), the [wisp.priv_directory](https://hexdocs.pm/wisp/wisp.html#priv_directory) function can be used to reliably get a path to this directory, allowing easy access to all files using [simplifile](https://hexdocs.pm/simplifile/index.html) or [wisp.serve_static](https://hexdocs.pm/wisp/wisp.html#serve_static), for example.\n\nIn general, a similar approach should be preferred when building for Javascript. But it is sometimes more useful to improve load times, and embed other resources directly into the generated bundle. Embedding certain build-time constants can also be used by bundlers to improve dead code elimintation/tree shaking. Please keep in mind that you may also be able to use [vite](https://vitejs.dev/guide/assets.html) or [esbuild](https://esbuild.github.io/content-types/#data-url) directly for this, which are both highly tested and extensible options.\n\nI personally use this package to embed build-time constants from Netlify into my Javascript bundle. Another good use case might be to apply additional pre-processing to you source files to generate more interesting source code (see the examples on how to do that).\n\n## Using the Gleam API\n\nSince you&#39;re likely going to depend on the generated modules, you cannot put code that uses `embeds` into the same project that you&#39;re trying to use the generated files in, not even in `test`! If you exclude the generated modules in `.gitignore`, or just want to delete and regenerate them all, your project will stop to build entirely, leaving you unable to run your code generator as well.\n\nThe common solution to this problem is to add another Gleam project to your repository for your build and codegen tools, and reference that new project using path dev dependencies in your main project(s).\n\nLet&#39;s assume you currently have a repository structure already looking like this:\n\n```text\n.\n├── server\n│   ├── gleam.toml\n│   ├── README.md\n│   ├── src\n│   └── test\n└── client\n    ├── gleam.toml\n    ├── README.md\n    ├── src\n    └── test\n```\n\nYou can add another project called `build`, and then add it as a dev dependency to all required projects by adding it to their `gleam.toml`:\n\n```toml\n[dev-dependencies]\nbuild = { path = &quot;../build&quot; }\n```\n\n&gt; **Hint:** Make sure to also run `gleam deps update` after modifying your `gleam.toml` manually, or adding dependencies to your `build` project!\n\nThis new `build` project does not require any codegen itself, and therefore can be built and cached without affecting any of your other projects.\n\nOverall your new structure should now look like this:\n\n```text\n.\n├── build\n├── server\n└── client\n```\n\nwhere you can now run\n\n```sh\ngleam run -m build\n```\n\nin the projects with the dev dependency added.\n","url":"index.html"},{"doc":"embeds","title":"embeds","content":"# Generating typed definitions from environment variables\n\nAn example of how to use `embeds/env` to generate constants of other types, like integers, boolean flags, or custom enums. Note that for simple cases like booleans or ints, command-line arguments can be used.\n\nYou can check out the full example project [here.](https://gitlab.com/arkandos/gleam-embeds/-/tree/main/examples/typed_env?ref_type=heads)\n\n## Running\n\nRun the project, providing some environment variables:\n\n```sh\nBUILD_ENV=production \\\nBUILD_KNOWN_BUGS=9001.000000004 \\\nBUILD_ENABLE_AWESOMENESS=YES \\\nBUILD_REVISION=16 \\\ngleam run\n```\n\n## Full Example Source\n\n&gt; **NOTE:** In a real project, you would add a seperate `build` path dev dependency.\nThis avoids issues when regenerating files that your project already depends on.\n\n```gleam\nimport embeds/env\nimport embeds/generate\nimport gleam/io\nimport gleam/option\n\npub fn main() {\n  let result =\n    env.generate(env.Options(\n      module: &quot;build_env&quot;,\n      prefix: &quot;BUILD_&quot;,\n      regex: option.None,\n      parse: env.typed_parse([\n        #(&quot;BUILD_ENV&quot;, environment),\n        #(&quot;BUILD_ENABLE_AWESOMENESS&quot;, env.bool),\n        // needs to be float because otherwise the login email doesn&#39;t work\n        #(&quot;BUILD_KNOWN_BUGS&quot;, env.float),\n        #(&quot;BUILD_REVISION&quot;, env.string),\n      ]),\n      // we need to include the environment type we want to use somehow -\n      // right here we just put it onto the top of the generated file.\n      // an alternative might be to put this type into a `shared` project.\n      header: &quot;\n        pub type Environment {\n          Development\n          Testing\n          Production\n        }\n      &quot;,\n    ))\n\n  case result {\n    Ok(Nil) -&gt; Nil\n    Error(errs) -&gt; io.println_error(generate.describe_errors(errs))\n  }\n}\n\n// a custom environment parser\nfn environment(value: String) -&gt; generate.Code {\n  // requires us to set the header above to include the type!\n  case value {\n    &quot;development&quot; -&gt; generate.Constant(&quot;Development&quot;)\n    &quot;production&quot; -&gt; generate.Constant(&quot;Production&quot;)\n    &quot;testing&quot; -&gt; generate.Constant(&quot;Testing&quot;)\n    _ -&gt; generate.GenerateError(&quot;Invalid environment: &quot; &lt;&gt; value)\n  }\n}\n```\n","url":"examples/typed-env.html"},{"doc":"embeds","title":"embeds","content":"# Embed images as Data URLs\n\nA demo of how to customize `embeds/files` to generate [data URLs](https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/Data_URLs) instead of raw constants instead.\n\nIt uses the [file_type](https://hexdocs.pm/file_type/readme.html) Elxir library to detect the mime types of files, so it requires Elixir to be installed.\n\nThe `img/lucypride.png` is a downscaled PNG based on the original SVG from the [the Gleam website repository](https://github.com/gleam-lang/website/raw/main/images/lucy/lucypride.svg).\n\nYou can check out the full example project [here.](https://gitlab.com/arkandos/gleam-embeds/-/tree/main/examples/data_urls?ref_type=heads)\n\n## Example source code\n\nRunning this project will produce a `images/lucy.gleam` module!\n\n&gt; **NOTE:** In a real project, you would add a seperate `build` path dev dependency.\nThis avoids issues when regenerating files that your project already depends on.\n\n```gleam\nimport embeds/files\nimport embeds/generate\nimport gleam/bit_array\nimport gleam/io\nimport gleam/option\nimport gleam/result\nimport gleam/string\n\npub fn main() {\n  let result =\n    files.generate(files.Options(\n      // where to get files from?\n      sources: [\n        files.Source(\n          // take all files from ./img (relative to the project root)\n          src: &quot;./img&quot;,\n          // generate a module called &quot;images.gleam&quot;\n          module: &quot;images&quot;,\n          // respect the directory structure\n          flatten: files.DontFlatten,\n          // regardless of how deep the directory tree is\n          max_depth: option.None,\n          // and filter out hidden files\n          filter: files.default_filter,\n        ),\n      ],\n      // how to convert a file into Gleam source code?\n      print: fn(path, data) {\n        // skip files if we can&#39;t detect the mime type,\n        // generate a Gleam constant otherwise\n        generate.skip_or_const({\n          use #(_extension, mime_type) &lt;- result.map(get_file_type(path))\n          // build a data url\n          let bits = generate.data_to_bits(data)\n          let base64 = bit_array.base64_url_encode(bits, True)\n          let data_url = &quot;data:&quot; &lt;&gt; mime_type &lt;&gt; &quot;;base64,&quot; &lt;&gt; base64\n          // convert the string to Gleam source code\n          string.inspect(data_url)\n        })\n      },\n      header: generate.default_header,\n      // Always regenerate\n      force: True,\n    ))\n\n  case result {\n    Ok(Nil) -&gt; Nil\n    Error(errs) -&gt; io.println_error(generate.describe_errors(errs))\n  }\n}\n\n// -- EXTERNALS\n\nfn get_file_type(path: String) -&gt; Result(#(String, String), Nil) {\n  // make sure we throw away the Elixir error value\n  do_get_file_type(path) |&gt; result.nil_error\n}\n\n// we use the file_type elixir library to detect mime types.\n// NOTE: the error type is wrong here, be careful!\n@external(erlang, &quot;Elixir.FileType&quot;, &quot;from_path&quot;)\nfn do_get_file_type(path: String) -&gt; Result(#(String, String), Nil)\n```\n","url":"examples/data-urls.html"},{"doc":"embeds","title":"embeds","content":"# Embedding SVGs into Lustre apps\n\nA demo of how to use `embeds/files` and [html_lustre_converter](https://hexdocs.pm/html_lustre_converter) together.\n\nIt contains a very simple [lustre](https://hexdocs.pm/lustre/index.html) demo app, showing our favorite Starfish!\n\nThe build script is located in `test/build.gleam`, as suggested on the Discord.\n\nThe `icons/lucy.svg` was taken from [the Gleam website](https://gleam.run/images/lucy/lucy.svg), with the `width` and `height` attributes removed for demo purposes.\n\nYou can check out the full example project [here.](https://gitlab.com/arkandos/gleam-embeds/-/tree/main/examples/lustre_icons?ref_type=heads)\n\n## Running\n\nSince `html_lustre_converter` depends on [javascript_dom_parser](https://hexdocs.pm/javascript_dom_parser/),\nonly the Deno runtime is supported right now.\n\n```sh\ngleam run -m build\ngleam run -m lustre/dev start\n```\n\n## `build.gleam` source code\n\n**NOTE:** In a real project, you would add a seperate `build` path dev dependency.\nThis avoids issues when regenerating files that your project already depends on.\n\n```gleam\nimport embeds/files\nimport embeds/generate\nimport gleam/io\nimport gleam/option\nimport gleam/string\nimport html_lustre_converter\nimport javascript_dom_parser/deno_polyfill\n\npub fn main() {\n  // required by html_lustre_converter\n  deno_polyfill.install_polyfill()\n\n  let result =\n    files.generate(files.Options(\n      // where to get files from?\n      sources: [\n        files.Source(\n          // take all files from ./icons (relative to the project root)...\n          src: &quot;./icons&quot;,\n          // generate a module called &quot;icons.gleam&quot;...\n          module: &quot;icons&quot;,\n          // put all found icons in the same module...\n          flatten: files.FlattenAll,\n          // regardless of how deep the directory tree is...\n          max_depth: option.None,\n          // and we are only interested in *.svg files...\n          filter: fn(_is_dir, path) {\n            // we are only interested in svg files.\n            string.ends_with(path, &quot;.svg&quot;)\n          },\n        ),\n      ],\n      // how to convert a file into Gleam source code?\n      print: fn(path, data) {\n        // if we get a svg text file, convert it using html_lustre_converter.\n        // NOTE: we technically already filtered in sources,\n        // so this check is reduntant currently!\n        case string.ends_with(path, &quot;.svg&quot;), data {\n          True, generate.Text(svg) -&gt;\n            generate.FunctionBody(html_lustre_converter.convert(svg))\n          _, _ -&gt; generate.Skip\n        }\n      },\n      // add the required imports\n      header: string.join(\n        [\n          &quot;import lustre/element/svg&quot;,\n          // html_lustre_converter generates attribute calls without namespaces\n          &quot;import lustre/attribute.{attribute}&quot;,\n        ],\n        &quot;\\n&quot;,\n      ),\n      // only regenerate the module if our icons have changed.\n      force: False,\n    ))\n\n  case result {\n    Ok(Nil) -&gt; Nil\n    Error(errs) -&gt; io.println_error(generate.describe_errors(errs))\n  }\n}\n```\n","url":"examples/lustre-icons.html"},{"doc":"embeds/generate","title":"Code","content":"pub type Code {\n  Skip\n  GenerateError(String)\n  Constant(String)\n  FunctionBody(String)\n}\n Code represents the rendered string of some Gleam expression.\n\n Because some expressions can&#39;t be constants, you can also choose to print\n function bodies instead.\nSkip\n Skip generating code for this symbol.\n\nGenerateError(String)\n Codegen produced some error that should be reported to the user.\n\nConstant(String)\n Produce a constant definition in Gleam, like\n \n pub const var_name = ...\n \n\nFunctionBody(String)\n Produce a zero-argument function definition in Gleam, like\n \n pub fn var_name() {\n     ...\n }\n \n You can use this constructor instead of `Constant` when your code contains\n anonymous functions, function calls, or other expressions that are not\n allowed inside of constants.\n\nSynonyms:\nembeds/generate.Code\nembeds/generate Code","url":"embeds/generate.html#Code"},{"doc":"embeds/generate","title":"Data","content":"pub type Data {\n  Text(String)\n  Binary(BitArray)\n}\n Data represents the contents of some file or other external source,\n and can either be UTF-8 `Text`, or `Binary`.\nText(String)\n\n\nBinary(BitArray)\n\n\nSynonyms:\nembeds/generate.Data\nembeds/generate Data","url":"embeds/generate.html#Data"},{"doc":"embeds/generate","title":"Definition","content":"pub type Definition {\n  Definition(\n    name: String,\n    read: fn() -&gt; Result(Data, String),\n    last_modified_seconds: Int,\n    module: String,\n    var_name: String,\n  )\n}\n A single codegen unit, potentially producing a single `Code` statement\n (so either a constant definition or a function.)\nDefinition(\n  name: String,\n  read: fn() -&gt; Result(Data, String),\n  last_modified_seconds: Int,\n  module: String,\n  var_name: String,\n)\n\nname\n&lt;p&gt;A human-identifyable &lt;code&gt;name&lt;/code&gt; for this definition used in error messages.\nUsually the file path, or the name of the environment variable.&lt;/p&gt;\n\nread\n&lt;p&gt;Files might get big, and definitions in general might be fetched from anywhere,\nso reading is deferred using this closure.&lt;/p&gt;\n\nlast_modified_seconds\n&lt;p&gt;A unix timestamp indicating the last modification time in seconds since\nJan 01, 1970 for the source file of this definition.&lt;/p&gt;\n&lt;p&gt;If a module only contains definitions that were modified before the\nmodule was last generated, generation will be skipped.&lt;/p&gt;\n&lt;p&gt;You can savely just set this value to 0 if you don’t have a good value\navailable.&lt;/p&gt;\n\nmodule\n&lt;p&gt;The name of the module this definition belongs to.\nNote that no sanitization takes place, and &lt;code&gt;module&lt;/code&gt; should be a valid\nGleam module name.&lt;/p&gt;\n&lt;p&gt;You can use &lt;code&gt;to_gleam_module_name&lt;/code&gt; to turn a path or arbitrary string\ninto a valid Gleam module path.&lt;/p&gt;\n\nvar_name\n&lt;p&gt;The name of the target constant or function name.\nNote that no sanitization takes place, and &lt;code&gt;var_name&lt;/code&gt; should be a valid\nGleam identifier.&lt;/p&gt;\n&lt;p&gt;You can use &lt;code&gt;to_gleam_identifier&lt;/code&gt; to turn an arbitrary string into a\nvalid Gleam identifier.&lt;/p&gt;\n\nSynonyms:\nembeds/generate.Definition\nembeds/generate Definition","url":"embeds/generate.html#Definition"},{"doc":"embeds/generate","title":"Error","content":"pub type Error {\n  NotInsideGleamProject\n  InvalidArgument(name: String, value: String, reason: String)\n  CouldNotListFiles(path: String, FileError)\n  CouldNotRead(name: String, message: String)\n  DuplicateVarName(String, String)\n  PrinterError(name: String, message: String)\n  WriteError(path: String, FileError)\n}\n the internal Error type returned by all embeds functions.\nNotInsideGleamProject\n the current working directory is not a Gleam project, and has no `gleam.toml` file.\n\nInvalidArgument(name: String, value: String, reason: String)\n a command-line argument could not be parsed or is invalid.\n\nCouldNotListFiles(path: String, FileError)\n A simplifile error occurred while listing all files.\n\nCouldNotRead(name: String, message: String)\n An error occurred while calling the `read` function for a definition.\n\nDuplicateVarName(String, String)\n Multiple definitions would produce the same constant in the same module.\n\nPrinterError(name: String, message: String)\n The printer returned an error that should be reported.\n\nWriteError(path: String, FileError)\n A simplifile error occurred while writing to the output module file.\n\nSynonyms:\nembeds/generate.Error\nembeds/generate Error","url":"embeds/generate.html#Error"},{"doc":"embeds/generate","title":"default_header","content":"pub const default_header: String\n A simple module comment which is used as the header by the CLI\nSynonyms:\nembeds/generate.default_header\nembeds/generate default_header","url":"embeds/generate.html#default_header"},{"doc":"embeds/generate","title":"bits_to_data","content":"pub fn bits_to_data(bits: BitArray) -&gt; Data\n Try to convert a BitArray to `Text` first , returning `Binary` if the\n conversion fails.\nSynonyms:\nembeds/generate.bits_to_data\nembeds/generate bits_to_data","url":"embeds/generate.html#bits_to_data"},{"doc":"embeds/generate","title":"data_to_bits","content":"pub fn data_to_bits(data: Data) -&gt; BitArray\n Convert some data back to a BitArray.\nSynonyms:\nembeds/generate.data_to_bits\nembeds/generate data_to_bits","url":"embeds/generate.html#data_to_bits"},{"doc":"embeds/generate","title":"default_print","content":"pub fn default_print(name: String, data: Data) -&gt; Code\n convert data to code, by simply calling string.inspect/bit_array.inspect.\nSynonyms:\nembeds/generate.default_print\nembeds/generate default_print","url":"embeds/generate.html#default_print"},{"doc":"embeds/generate","title":"describe_error","content":"pub fn describe_error(err: Error) -&gt; String\n Turn an `Error` into a human-readable string.\nSynonyms:\nembeds/generate.describe_error\nembeds/generate describe_error","url":"embeds/generate.html#describe_error"},{"doc":"embeds/generate","title":"describe_errors","content":"pub fn describe_errors(errs: List(Error)) -&gt; String\n Turn multiple erros into a human-readable string.\nSynonyms:\nembeds/generate.describe_errors\nembeds/generate describe_errors","url":"embeds/generate.html#describe_errors"},{"doc":"embeds/generate","title":"generate","content":"pub fn generate(\n  defs: List(Definition),\n  header: String,\n  print: fn(String, Data) -&gt; Code,\n  force: Bool,\n) -&gt; Result(Nil, List(Error))\n Generates module files based on a list of Definitions.\nSynonyms:\nembeds/generate.generate\nembeds/generate generate","url":"embeds/generate.html#generate"},{"doc":"embeds/generate","title":"guard_gleam_project","content":"pub fn guard_gleam_project(\n  f: fn() -&gt; Result(a, List(Error)),\n) -&gt; Result(a, List(Error))\n enforce that the current working directory is a gleam project\n (by checking a gleam.toml exists).\n Automatically called by `generate`, but you may wish to exit earlier.\n Designed to be used with `use`.\nSynonyms:\nembeds/generate.guard_gleam_project\nembeds/generate guard_gleam_project","url":"embeds/generate.html#guard_gleam_project"},{"doc":"embeds/generate","title":"result_to_const","content":"pub fn result_to_const(result: Result(String, String)) -&gt; Code\n Helper function to convert a Result into either a `Constant` or `GenerateError`.\nSynonyms:\nembeds/generate.result_to_const\nembeds/generate result_to_const","url":"embeds/generate.html#result_to_const"},{"doc":"embeds/generate","title":"result_to_fn","content":"pub fn result_to_fn(result: Result(String, String)) -&gt; Code\n Helper function to convert a Result into either a `FunctionBody` or `GenerateError`.\nSynonyms:\nembeds/generate.result_to_fn\nembeds/generate result_to_fn","url":"embeds/generate.html#result_to_fn"},{"doc":"embeds/generate","title":"skip_or_const","content":"pub fn skip_or_const(result: Result(String, Nil)) -&gt; Code\n Helper function to convert a Result into either a `Constant` or skip generating on error.\nSynonyms:\nembeds/generate.skip_or_const\nembeds/generate skip_or_const","url":"embeds/generate.html#skip_or_const"},{"doc":"embeds/generate","title":"skip_or_fn","content":"pub fn skip_or_fn(result: Result(String, Nil)) -&gt; Code\n Helper function to convert a Result into either a `FunctionBody` or skip generating on error.\nSynonyms:\nembeds/generate.skip_or_fn\nembeds/generate skip_or_fn","url":"embeds/generate.html#skip_or_fn"},{"doc":"embeds/generate","title":"to_gleam_identifier","content":"pub fn to_gleam_identifier(str: String) -&gt; String\n Ensures the given string is a valid Gleam identifier by replacing invalid\n characters by underscores.\n\n Empty strings return &quot;empty&quot;, numbers will be prefixed with &quot;n&quot;.\n\n If the resulting variable name would be a reserved keyword, an additional\n underscore will be added at the end.\nSynonyms:\nembeds/generate.to_gleam_identifier\nembeds/generate to_gleam_identifier","url":"embeds/generate.html#to_gleam_identifier"},{"doc":"embeds/generate","title":"to_gleam_module_name","content":"pub fn to_gleam_module_name(path: String) -&gt; String\n Ensures a path is a valid Gleam module path, by making sure that every piece\n is a valid Gleam identifier.\n\n Unix `/` and Windows `\\\\` are both treated as module separators.\n\n **NOTE**:The top-level module is not allowed to be called `gleam`, but this\n  function does not enforce this!\nSynonyms:\nembeds/generate.to_gleam_module_name\nembeds/generate to_gleam_module_name","url":"embeds/generate.html#to_gleam_module_name"},{"doc":"embeds/generate","title":"embeds/generate","content":" This module exposes the internal/shared code for both `embeds/files` as\n well as `embeds/env`.\n\n It exposes some common type aliases and functions, and provides a\n lower-level API to generate module files from a set of `Definitions`.\n\n I recommend starting with the `embeds/files` and/or `embeds/env` modules\n first before diving into this one!","url":"embeds/generate.html"},{"doc":"embeds/env","title":"Options","content":"pub type Options {\n  Options(\n    module: String,\n    header: String,\n    parse: fn(String, String) -&gt; Code,\n    prefix: String,\n    regex: Option(Regex),\n  )\n}\n\nOptions(\n  module: String,\n  header: String,\n  parse: fn(String, String) -&gt; Code,\n  prefix: String,\n  regex: Option(Regex),\n)\n\nmodule\n&lt;p&gt;Name of the module to generate.&lt;/p&gt;\n&lt;p&gt;Note that the module name is not sanitized, and should represent a valid\nGleam module name.\nYou can use &lt;code&gt;generate.to_gleam_module_name&lt;/code&gt; to turn a path or arbitrary\ninto a valid value.&lt;/p&gt;\n\nheader\n&lt;p&gt;The module header&lt;/p&gt;\n\nprefix\n&lt;p&gt;Only environment variables matching this prefix will be included,\nand the prefix will be stripped from the variable name.&lt;/p&gt;\n\nregex\n&lt;p&gt;Optional regex that needs to match the variable name after the &lt;code&gt;prefix&lt;/code&gt;\nhas been stripped.\nIf a capture group exists, the first match will be used for the environment variable.&lt;/p&gt;\n\nSynonyms:\nembeds/env.Options\nembeds/env Options","url":"embeds/env.html#Options"},{"doc":"embeds/env","title":"Parser","content":"pub type Parser =\n  fn(String) -&gt; Code\n A parser is just a function turning an environment variable into Code.\n\n ## Custom parsers\n \n \n type Environment {\n     Development\n     Testing\n     Production\n }\n \n fn environemnt(value: String) -&gt; Code {\n     // you need to make sure the type constructors are available,\n     // for example by adding them to the header!\n     case value {\n         &quot;development&quot; -&gt; generate.Constant(string.inspect(Development))\n         &quot;production&quot; -&gt; generate.Constant(string.inspect(Production))\n         &quot;testing&quot; -&gt; generate.Constant(string.inspect(Testing))\n         _ -&gt; generate.GenerateError(&quot;Invalid Environment: &quot; &lt;&gt; value)\n     }\n }\n \n\nSynonyms:\nembeds/env.Parser\nembeds/env Parser","url":"embeds/env.html#Parser"},{"doc":"embeds/env","title":"bool","content":"pub fn bool(value: String) -&gt; Code\n A parser for boolean flags\nSynonyms:\nembeds/env.bool\nembeds/env bool","url":"embeds/env.html#bool"},{"doc":"embeds/env","title":"default_parse","content":"pub fn default_parse(key: String, value: String) -&gt; Code\n A default parse function, converting environment variables into\n string constants.\nSynonyms:\nembeds/env.default_parse\nembeds/env default_parse","url":"embeds/env.html#default_parse"},{"doc":"embeds/env","title":"float","content":"pub fn float(value: String) -&gt; Code\n A parser for floating-point literals\nSynonyms:\nembeds/env.float\nembeds/env float","url":"embeds/env.html#float"},{"doc":"embeds/env","title":"generate","content":"pub fn generate(options: Options) -&gt; Result(Nil, List(Error))\n Generate Gleam modules, containing constants for all found variables.\n Exported for easier integration with other build tools.\n\n ## Example\n\n \n import embeds/env\n import embeds/generate\n import gleam/option\n import gleam/io\n\n pub fn main() {\n     let result = env.generate(env.Options(\n         module: &quot;build_env&quot;,\n         header: generate.default_header,\n         parse: env.default_parse,\n         prefix: &quot;BUILD_&quot;,\n         regex: option.None,\n     ))\n\n     case result {\n        Ok(Nil) -&gt; Nil\n        Error(errs) -&gt; io.println_error(generate.describe_errors(errs))\n     }\n }\n \nSynonyms:\nembeds/env.generate\nembeds/env generate","url":"embeds/env.html#generate"},{"doc":"embeds/env","title":"get_variables","content":"pub fn get_variables(\n  module: String,\n  prefix: String,\n  regex: Option(Regexp),\n) -&gt; List(Definition)\n Get all variables from the environment matching a prefix and a regex,\n and convert them into definitions.\nSynonyms:\nembeds/env.get_variables\nembeds/env get_variables","url":"embeds/env.html#get_variables"},{"doc":"embeds/env","title":"int","content":"pub fn int(value: String) -&gt; Code\n A parser for integers literals\nSynonyms:\nembeds/env.int\nembeds/env int","url":"embeds/env.html#int"},{"doc":"embeds/env","title":"list","content":"pub fn list(\n  item_parser: fn(String) -&gt; Code,\n  separator: String,\n) -&gt; fn(String) -&gt; Code\n A parser for lists.\n Common separators might be `:` or `,`\nSynonyms:\nembeds/env.list\nembeds/env list","url":"embeds/env.html#list"},{"doc":"embeds/env","title":"main","content":"pub fn main() -&gt; Nil\n runs generate using command line arguments to fill out the arguments.\nSynonyms:\nembeds/env.main\nembeds/env main","url":"embeds/env.html#main"},{"doc":"embeds/env","title":"string","content":"pub fn string(value: String) -&gt; Code\n A string parser, just passing through the value\nSynonyms:\nembeds/env.string\nembeds/env string","url":"embeds/env.html#string"},{"doc":"embeds/env","title":"typed_parse","content":"pub fn typed_parse(\n  definitions: List(#(String, fn(String) -&gt; Code)),\n) -&gt; fn(String, String) -&gt; Code\n A function taking a list of `(key, parser)` pairs, and turning them\n into a parse function to be used in `Options`.\n \n This allows you to generate other types of variables, instead of just strings.\n \n ## Example\n \n \n let options = env.Options(\n     module: &quot;build_env&quot;,\n     header: generate.default_header,\n     prefix: &quot;BUILD_&quot;,\n     regex: option.None,\n     parse: env.typed_parse([\n         #(&quot;BUILD_ENV&quot;, string),\n         #(&quot;BUILD_REVISION&quot;, int),\n         #(&quot;BUILD_ENABLE_AWESOMENESS&quot;, bool)\n     ]),\n )\n \nSynonyms:\nembeds/env.typed_parse\nembeds/env typed_parse","url":"embeds/env.html#typed_parse"},{"doc":"embeds/env","title":"typed_parse_fallback","content":"pub fn typed_parse_fallback(\n  definitions: List(#(String, fn(String) -&gt; Code)),\n  fallback: fn(String, String) -&gt; Code,\n) -&gt; fn(String, String) -&gt; Code\n A variant of `typed_parse` that takes an additional fallback function\n used when the variable is not found inside of `definitions`.\n \n Using `default_parse` as the fallback is useful to include all\n variables not found in `definitions` as simple string constants.\nSynonyms:\nembeds/env.typed_parse_fallback\nembeds/env typed_parse_fallback","url":"embeds/env.html#typed_parse_fallback"},{"doc":"embeds/env","title":"embeds/env","content":" `embeds/env` generates a Gleam module containing constants and zero-argument\n functions based on environment variables.\n\n This module is intended to be run as a CLI tool during your build process:\n ```sh\n gleam run -m embeds/env -- --help\n ```\n\n The functions exposed in this module can also be used directly to support\n advanced use cases inside custom pre-build scripts.","url":"embeds/env.html"},{"doc":"embeds/files","title":"Flatten","content":"pub type Flatten {\n  DontFlatten\n  FlattenAll\n  FlattenFirst(Int)\n}\n\nDontFlatten\n Don&#39;t flatten. Every directory will generate another sub-module.\n\nFlattenAll\n Flatten all files, resulting in a single Gleam module\n\nFlattenFirst(Int)\n Only flatten the first N directory levels\n\nSynonyms:\nembeds/files.Flatten\nembeds/files Flatten","url":"embeds/files.html#Flatten"},{"doc":"embeds/files","title":"Options","content":"pub type Options {\n  Options(\n    sources: List(Source),\n    header: String,\n    print: fn(String, Data) -&gt; Code,\n    force: Bool,\n  )\n}\n\nOptions(\n  sources: List(Source),\n  header: String,\n  print: fn(String, Data) -&gt; Code,\n  force: Bool,\n)\n\nsources\n&lt;p&gt;A list of sources to scan for files.\nSee the &lt;code&gt;Source&lt;/code&gt; documentation for more info.&lt;/p&gt;\n\nheader\n&lt;p&gt;The header to place on the top of all modules&lt;/p&gt;\n\nprint\n&lt;p&gt;A printer is a function that turns a file into some Gleam code.\nYou can use &lt;code&gt;generate.default_print&lt;/code&gt; to produce String or BitArray literals.&lt;/p&gt;\n\nforce\n&lt;p&gt;If force is enabled, all modules will be regenerated, regardless of whether\nor not their timestamp has changed.&lt;/p&gt;\n\nSynonyms:\nembeds/files.Options\nembeds/files Options","url":"embeds/files.html#Options"},{"doc":"embeds/files","title":"Source","content":"pub type Source {\n  Source(\n    src: String,\n    module: String,\n    flatten: Flatten,\n    max_depth: Option(Int),\n    filter: fn(Bool, String) -&gt; Bool,\n  )\n}\n A Source, describing how to map a directory structure to Gleam module names.\nSource(\n  src: String,\n  module: String,\n  flatten: Flatten,\n  max_depth: Option(Int),\n  filter: fn(Bool, String) -&gt; Bool,\n)\n\nsrc\n&lt;p&gt;The source directory, where your asset files live.&lt;/p&gt;\n\nmodule\n&lt;p&gt;The base name of the generated module.&lt;/p&gt;\n&lt;p&gt;The module name will be used as a base for all files in &lt;code&gt;src&lt;/code&gt;, and nested\ndirectories will generate nested modules. (see &lt;code&gt;flatten&lt;/code&gt; on how to tweak this)\nMultiple sources can re-use the same module name.&lt;/p&gt;\n\nflatten\n&lt;p&gt;how many levels of directories to ignore before starting to group things into nested modules,\nwhich is sometimes useful to “skip” some diretories first.&lt;/p&gt;\n\nmax_depth\n&lt;p&gt;The maximum amount of levels to traverse before stopping looking for files.\nThis setting is independent of &lt;code&gt;flatten&lt;/code&gt; and will instead work based on the directory tree.\nA value of 0 means to only look at the given &lt;code&gt;src&lt;/code&gt; directory.\nA None value represents no limit.&lt;/p&gt;\n\nfilter\n&lt;p&gt;A filter function called on file and directory names.\nIf the function returns &lt;code&gt;False&lt;/code&gt;, the directory or file will be skipped.\nSet to &lt;code&gt;default_filter&lt;/code&gt; to filter out hidden files and directories.&lt;/p&gt;\n&lt;p&gt;The first argument is set to &lt;code&gt;True&lt;/code&gt; if the given path is a directory.&lt;/p&gt;\n\nSynonyms:\nembeds/files.Source\nembeds/files Source","url":"embeds/files.html#Source"},{"doc":"embeds/files","title":"default_filter","content":"pub fn default_filter(is_dir: Bool, path: String) -&gt; Bool\n A default filter function to use inside sources.\n Filters out hidden files and directories.\nSynonyms:\nembeds/files.default_filter\nembeds/files default_filter","url":"embeds/files.html#default_filter"},{"doc":"embeds/files","title":"generate","content":"pub fn generate(options: Options) -&gt; Result(Nil, List(Error))\n Generate Gleam modules, containing constants for all found files, given the sources.\n Exported for easier integration with other build tools.\n\n ## Example\n\n \n import embeds/files\n import embeds/generate\n import gleam/option\n import gleam/io\n\n pub fn main() {\n     let result = files.generate(files.Options(\n         sources: [files.Source(\n             src: &quot;./assets&quot;,\n             module: &quot;assets&quot;,\n             flatten: files.FlattenAll,\n             max_depth: option.None,\n             filter: files.default_filter,\n         )],\n\n         print: generate.default_print,\n         header: generate.default_header,\n         force: False,\n     ))\n\n     case result {\n        Ok(Nil) -&gt; Nil\n        Error(errs) -&gt; io.println_error(generate.describe_errors(errs))\n     }\n }\n \nSynonyms:\nembeds/files.generate\nembeds/files generate","url":"embeds/files.html#generate"},{"doc":"embeds/files","title":"get_files","content":"pub fn get_files(\n  src: Source,\n) -&gt; Result(List(Definition), FileError)\n Find all files inside a `Source`.\n You usually do not need to call this function yourself.\nSynonyms:\nembeds/files.get_files\nembeds/files get_files","url":"embeds/files.html#get_files"},{"doc":"embeds/files","title":"main","content":"pub fn main() -&gt; Nil\n runs generate using command line arguments to fill out the arguments.\nSynonyms:\nembeds/files.main\nembeds/files main","url":"embeds/files.html#main"},{"doc":"embeds/files","title":"embeds/files","content":" `embeds/files` generates Gleam modules containing constants and zero-argument\n functions based on the contents of files.\n\n This module is intended to be run as a CLI tool during your build process:\n ```sh\n gleam run -m embeds/files -- --help\n ```\n\n The functions exposed in this module can also be used directly to support\n advanced use cases inside custom pre-build scripts.","url":"embeds/files.html"}]);